/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * include/config.h: compile time configuration header file
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: config.h,v 1.75 2002/12/22 18:11:44 rageircd Exp $
 */

#ifndef	__config_include__
#define	__config_include__

#include "setup.h"

#ifdef OS_SUNOS_DBE
#define FD_SETSIZE HARD_FDLIMIT_
#define NOFILE HARD_FDLIMIT_
#endif

#ifdef OS_SOLARIS2
#define NO_PRIORITY
#else
#undef NO_PRIORITY
#endif

/*
 * Files and Directories
 * IRCD_PREFIX is set using ./configure --prefix=PREFIX
 *
 * Apologies for OS_CYGWIN shit here, we'll work something
 * better out further on down the line. :/
 */
#ifdef OS_CYGWIN
#define PDIR		IRCD_PREFIX "\\"
#define LDIR		PDIR "logs\\"
#define IRCD_EXE	PDIR "bin\\ircd"
#define RESOLV_FILE	PDIR "ircd.resolv"
#else
#define PDIR		""
#define LDIR		"logs/"
#define IRCD_EXE	IRCD_PREFIX "/bin/ircd"
#define RESOLV_FILE	"/etc/resolv.conf"
#endif

#define CONFIG_FILE	PDIR "ircd.conf"
#define MOTD_FILE	PDIR "ircd.motd"
#define OPERMOTD_FILE	PDIR "ircd.opermotd"
#define RULES_FILE	PDIR "ircd.rules"
#define STATS_FILE	PDIR "ircd.stats"
#define PID_FILE	PDIR "ircd.pid"
#define ENTROPY_FILE	PDIR "ircd.entropy"

#ifdef HAVE_OPENSSL_ON
#define SSL_CERT_FILE	PDIR "cert.pem"
#define SSL_KEY_FILE	PDIR "key.pem"
#endif

#define LOG_DEFAULT	LDIR "ircd.log"
#define LOG_ERROR	LDIR "error.log"
#define LOG_KILL	LDIR "kill.log"
#define LOG_BAN		LDIR "ban.log"
#define LOG_CLIENT	LDIR "client.log"
#define LOG_SERVER	LDIR "server.log"
#define LOG_OPER	LDIR "oper.log"
#define LOG_DCCALLOW	LDIR "dccallow.log"
#define LOG_ADNS	LDIR "adns.log"
#define LOG_DEBUG	LDIR "debug.log"

/*
 * Service Client Nicknames
 *
 * These are used for the service alias functions, and should be changed
 * according to the services available on your network. These were
 * dynamic configuration options, however, I fail to see why anyone
 * would want to change them anyway.
 */
#define CHANSERV "ChanServ"
#define NICKSERV "NickServ"
#define MEMOSERV "MemoServ"
#define OPERSERV "OperServ"
#define STATSERV "StatServ"

/*
 * Memory Pre-allocation
 *
 * These determine how many structures memory should be pre-allocated for
 * at bootup, used with the block allocator. All these values were
 * originally set to 1024 for EFNet, but we now use a quater of that,
 * as I cant see a network as large as EFNet using RageIRCd.
 */
#define LINKS_PREALLOCATE 256
#define CHANMEMBERS_PREALLOCATE 256
#define CLIENTS_PREALLOCATE 256
#define CHANNELS_PREALLOCATE 256
#define BANS_PREALLOCATE 256
#define THROTTLES_PREALLOCATE 256
#define HASHENTS_PREALLOCATE 256

/*
 * INITIAL_CLIENT_DBUFS and INITIAL_SERVER_DBUFS
 *
 * These define how many DBuf structures should be pre-allocated when
 * the server is first started. DBuf's are the buffers that handle the
 * data to and from the server to connections. Pre-allocating these
 * means you do not have to allocate them every time a client connects.
 *
 * INITIAL_CLIENT_DBUFS - number of client dbufs to preallocate
 *	o) Recommended non-hub server value: 512
 *	o) Recommended hub server value: 256
 * INITIAL_SERVER_DBUFS - number of server dbufs to preallocate
 *	o) Recommended non-hub server value: 16
 *	o) Recommended hub server value: 32
 */
#ifdef HAVE_HUB_ON
#define INITIAL_CLIENT_DBUFS 256
#define INITIAL_SERVER_DBUFS 32
#else
#define INITIAL_CLIENT_DBUFS 512
#define INITIAL_SERVER_DBUFS 16
#endif

/*
 * Throttle Support
 *
 * THROTTLE_RECORDTIME - length to keep records for each ip (since last connect)
 * THROTTLE_HASHSIZE   - size of the throttle hash table
 *
 * We recommend you do not change these unless it's crutial.
 */
#define THROTTLE_RECORDTIME 1800
#define THROTTLE_HASHSIZE 25147

/*
 * BLOCKHEAP_GARBAGECOLLECT_FREQ
 *
 * If the block heap allocator is enabled, this is the time in seconds between
 * automatic memory garbage collection, which clean up block heaps to eliminate
 * fragmentation and memory leaks.
 */
#define BLOCKHEAP_GARBAGECOLLECT_FREQ (3600 * 3)

/* Don't change this... */
#define HARD_FDLIMIT	(HARD_FDLIMIT_ - 10)
#define MASTER_MAX	(HARD_FDLIMIT - MAX_BUFFER)

#include "defs.h"

/*
 * TS_MAX_DELTA and TS_WARN_DELTA - allowed delta for TS when another
 * server connects.
 * 
 * If the difference between my clock and the other server's clock is
 * greater than TS_MAX_DELTA, I send out a warning and drop the links.
 * 
 * If the difference is less than TS_MAX_DELTA, I just sends out a warning
 * but don't drop the link.
 * 
 * TS_MAX_DELTA currently set to 30 minutes to deal with older timedelta
 * implementation.  Once pre-hybrid5.2 servers are eradicated, we can
 * drop this down to 90 seconds or so. --Rodder
 */
#define TS_MAX_DELTA 120
#define TS_WARN_DELTA 15

/*
 * INCREASE_SOCK_BUFS
 *
 * If defined, the size of the send and receive socket buffers will be
 * dynamically allocated upon connection. Server buffers will then be
 * increased to 64k. However, client buffers shall remain at 8k.
 */
#define INCREASE_SOCK_BUFS

/*
 * LOG_LIBADNS_DEBUG
 *
 * Define this to log libadns debug messages to LOG_ADNS.
 */
#undef LOG_LIBADNS_DEBUG

/*
 * CLIENT_SERVER
 *
 * Changes behaviour of the HTM code to make clients lag less.
 */
#define CLIENT_SERVER

/*
 * IRCII_KLUDGE
 *
 * Define this will accomplish ircII standards by sending an extra
 * NOTICE at the beginning of client connections.
 */
#undef IRCII_KLUDGE

/*
 * NOISY_HTM_BY_DEFAULT
 *
 * Defines whether HTM should be set to NOISY by default.
 */
#define NOISY_HTM_BY_DEFAULT

/*
 * CLIENT_FLOOD
 *
 * This controls the number of bytes the server will allow a client
 * to send to the server without processing before the client is
 * dropped for flooding. Values greater than 8000 make no difference
 * to the server.
 */
#define	CLIENT_FLOOD 3500

/*
 * ANTINICKFLOOD_CHANGES and ANTINICKFLOOD_TIME
 *
 * These are the parameters for nickname changing flood protection.
 * Provided servinfo::options::anti-nick-flood is enabled, any nickname
 * changes will be blocked if the client in question has performed
 * ANTINICKFLOOD_CHANGES within ANTINICKFLOOD_TIME seconds.
 */
#define ANTINICKFLOOD_CHANGES 4
#define ANTINICKFLOOD_TIME 20

/*
 * MAX_AWAY_COUNT and MAX_AWAY_TIME
 *
 * These provide the parameters for the AWAY flood protection.
 * Provided servinfo::options::anti-away-flood is enabled, any client
 * attempting to do MAX_AWAY_COUNT AWAY message changes within
 * MAX_AWAY_TIME seconds will be ignored.
 */
#define MAX_AWAY_COUNT 5
#define MAX_AWAY_TIME 180

/* 
 * HTM_LOCK_ON_NETBURST Define this if you want your server to
 * stay in HTM while the server is synching to the network 
 * after a connect
*/
#undef HTM_LOCK_ON_NETBURST

/*
 * USE_NOQUIT use the noquit system to save bandwidth and network stress 
 * on netsplits. 
 */
#define USE_NOQUIT

/*
 * ALWAYS_SEND_DURING_SPLIT
 *
 * on a large network, if your server is carrying large amounts of clients,
 * and your server splits from the main network, the amount of allocated
 * dbufs will skyrocket as buffers fill up with QUIT messages. This code
 * attempts to combat this by sending out data whenever possible during a
 * split. - lucas
 */
#define ALWAYS_SEND_DURING_SPLIT

/*
 * CMDLINE_CONFIG
 *
 * Define this to enable the -c command-line option to specify an alternate
 * ircd.conf file.
 */
#define CMDLINE_CONFIG

/*
 * MAXCONNECTIONS
 *
 * WARNING: Do not change this directly. Change the HARD_FDLIMIT_
 * variable instead.
 *
 * Maximum number of network connections your server will allow.  This
 * should never exceed max. number of open file descrpitors and wont
 * increase this. Should remain LOW as possible. Most sites will
 * usually have under 30 or so connections. A busy hub or server may
 * need this to be as high as 50 or 60. Making it over 100 decreases
 * any performance boost gained from it being low. if you have a lot of
 * server connections, it may be worth splitting the load over 2 or
 * more servers. 1 server = 1 connection, 1 user = 1 connection. This
 * should be at *least* 3: 1 listen port, 1 dns port + 1 client
 */
#define MAXCONNECTIONS	HARD_FDLIMIT

/*
 * NICKNAMEHISTORYLENGTH
 *
 * This defines the length of the nickname history.  Each time a user
 * changes nickname or signs off, their old nickname is added to the
 * top of the list.
 *
 * WARNING: This is directly related to the amount of memory ircd will
 * use whilst resident and running - it hardly ever gets swapped to
 * disk! Memory will be preallocated for the entire whowas array when
 * ircd is started.
 */
#define NICKNAMEHISTORYLENGTH 1024

/*
 * TIMESEC - Time interval to wait and if no messages have been
 * received, then check for PINGFREQUENCY and CONNECTFREQUENCY
 */
#define TIMESEC 5

/*
 * DEFAULT_KLINE_EXPIRE_TIME
 *
 * Default time (in seconds) to set K:Line expire times to if
 * servinfo::options::write-perm-klines-to-file is enabled, but if
 * the server fails to open CONFIG_FILE.
 */
#define DEFAULT_KLINE_EXPIRE_TIME (3600 * 24)

/*
 * DEFAULT_PINGTIME
 *
 * Default amount of seconds before the server checks internal list
 * to see if any connections that have been inactive for a specified
 * amount of time are still alive.
 */
#define DEFAULT_PINGTIME 120

/*
 * DEFAULT_CONNECTFREQ
 *
 * Default amount of time to wait before attempting to reconnect
 * any broken links that have autoconnect in the link::options
 * section.
 */
#define DEFAULT_CONNECTFREQ 600

/*
 * DEFAULT_MAXCHANSPERUSER
 *
 * Default amount of channels a client is allowed to be in at any
 * one time.
 */
#define DEFAULT_MAXCHANSPERUSER 10

/*
 * DEFAULT_MAXCLIENTS
 *
 * Default amount of connections allowed in a class at any one time.
 */
#define DEFAULT_MAXCLIENTS 256

/*
 * DEFAULT_LINKDEPTH
 *
 * Default depth of a server link.
 */
#define DEFAULT_LINKDEPTH 5

/*
 * DEFAULT_CONNAUTH_TIMEOUT
 *
 * Default amount of time, in seconds, to cycle the internal resolver
 * list to expire un-resolved hosts.
 */
#define DEFAULT_CONNAUTH_TIMEOUT 3

/*
 * DEFAULT_PORTNUM
 *
 * This defines the default port number to use in configuration if the
 * specified port is out of range. Range is from 1 to 65535.
 */
#define DEFAULT_PORTNUM 7000

/*
 * HANGONGOODLINK and HANGONGOODLINK
 *
 * Often net breaks for a short time and it's useful to try to 
 * establishing the same connection again faster than DEFAULT_CONNECTFREQ
 * would allow. But, to keep trying on bad connection, we require that 
 * connection has been open for certain minimum time (HANGONGOODLINK) and 
 * we give the net few seconds to steady (HANGONRETRYDELAY). This latter 
 * has to be long enough that the other end of the connection has time to 
 * notice it broke too.
 */

#define HANGONRETRYDELAY 60
#define HANGONGOODLINK 3600

/*
 * WRITEWAITDELAY
 *
 * Number of seconds to wait for write to complete if stuck.
 */
#define WRITEWAITDELAY     10

/*
 * CONNECTTIMEOUT
 *
 * Number of seconds to wait for a connect(2) call to complete.
 *
 * NOTE: This must be at LEAST 10. When a client connects, it has
 * CONNECTTIMEOUT - 10 seconds for its host to respond to an identd
 * lookup and for a DNS answer to be retrieved.
 */
#define	CONNECTTIMEOUT	30

/*
 * KILLCHASETIMELIMIT
 *
 * Max time from the nickname change that still causes KILL automaticly 
 * to switch for the current nick of that user.
 */
#define KILLCHASETIMELIMIT 90

/*
 * FLUD - CTCP Flood Detection and Protection
 * 
 * This enables server CTCP flood detection and protection for local
 * clients. It works well against fludnets and flood clones.  The
 * effect of this code on server CPU and memory usage is minimal,
 * however you may not wish to take the risk, or be fundamentally
 * opposed to checking the contents of PRIVMSG's (though no privacy is
 * breached).  This code is not useful for routing only servers (ie,
 * HUB's with little or no local client base), and the hybrid team
 * strongly recommends that you do not use FLUD with HUB. The following
 * default thresholds may be tweaked, but these seem to work well.
 */
#ifndef HAVE_HUB_ON
#define FLUD
#endif

#ifdef FLUD
#define FLUD_NUM 4	/* Number of flud messages to trip alarm */
#define FLUD_TIME 3	/* Seconds in which FLUD_NUM msgs must occur */
#define FLUD_BLOCK 15	/* Seconds to block fluds */
#endif

/*
 * servinfo::option::anti-spambot-checks
 *
 * Don't touch these if you don't understand what is going on.
 * 
 * if a client joins MAX_JOIN_LEAVE_COUNT channels in a row, but spends
 * less than MIN_JOIN_LEAVE_TIME seconds on each one, flag it as a
 * possible spambot. disable JOIN for it and PRIVMSG but give no
 * indication to the client that this is happening. every time it tries
 * to JOIN OPER_SPAM_COUNTDOWN times, flag all opers on local server.
 * If a client doesn't LEAVE a channel for at least 2 minutes the
 * join/leave counter is decremented each time a LEAVE is done
 * 
 */
#define MIN_SPAM_NUM 5
#define MIN_SPAM_TIME 60
#define MIN_JOIN_LEAVE_TIME 60
#define MAX_JOIN_LEAVE_COUNT 25
#define OPER_SPAM_COUNTDOWN 5
#define JOIN_LEAVE_COUNT_EXPIRE_TIME 120

/*
 * If ANTI_SPAMBOT_WARN_ONLY is #define'd Warn opers about possible
 * spambots only, do not disable JOIN and PRIVMSG if possible spambot
 * is noticed Depends on your policies.
 */
#undef ANTI_SPAMBOT_WARN_ONLY

/*
 * If the OS has SOMAXCONN use that value, otherwise Use the value in
 * HYBRID_SOMAXCONN for the listen(); backlog try 5 or 25. 5 for AIX
 * and SUNOS, 25 should work better for other OS's
 */
#define HYBRID_SOMAXCONN 25

/* DO NOT EDIT ANYTHING BELOW THIS LINE! */

#undef DEBUGMODE

#ifdef APOLLO
#define RESTARTING_SYSTEMCALLS
#endif

#ifdef INCREASE_SOCK_BUFS
#define CLIENT_BUFFER_SIZE 4096
#define SEND_BUF_SIZE 2048
#endif

#if defined(DEBUGMODE) || defined(DNS_DEBUG)
extern void debug(int level, char *pattern, ...);
#define Debug(x) debug x
#define LOGFILE LOG_DEFAULT
#else
#define Debug(x)
#define LOGFILE "/dev/null"
#endif

#ifdef NOISY_HTM_BY_DEFAULT
#define NOISY_HTM_DEFAULT 1
#else
#define NOISY_HTM_DEFAULT 0
#endif

#ifdef HAVE_HUB_ON
#define LIFESUX_DEFAULT 0
#else
#define LIFESUX_DEFAULT 1
#endif

#define CONFIG_H_LEVEL_13

#endif /* __config_include__ */
