/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * include/dbuf.h: dbuf header file
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: dbuf.h,v 1.6 2002/10/25 22:42:21 rageircd Exp $
 */

#ifndef __dbuf_include__
#define __dbuf_include__

#ifndef __config_include__
#include "config.h"
#endif
#ifndef __dbuf_sys_types_h_include__
#define __dbuf_sys_types_h_include__
#include <sys/types.h>
#endif

#define DBUF_SIZE_CLIENT 2048
#define DBUF_SIZE_SERVER 65535

#define IsEol(x)	((x) == 10 || (x) == 13)
#define DBufLength(x)	((x)->length)
#define DBufClear(x)	dbuf_delete((x), DBufLength((x)))

typedef struct _dbuf DBuf;
typedef struct _dbuf_buffer DBufBuffer;
typedef struct _dbuf_block DBufBlock;

struct _dbuf {
	DBufBuffer *head, *tail;
	size_t length;
};

struct _dbuf_buffer {
	DBufBuffer *next;
	char *start;
	char *end;
	DBufBlock *parent;
	char data[1];
};

struct _dbuf_block {
	DBufBuffer *buf;
	unsigned int block_seq;
	int blocksize;
	int num;
	int given_back;
	int is_freeing;
	int (*callback)(int, int);
	DBufBlock *next;
};

extern int DBufCountC;
extern int DBufCountS;
extern int DBufUsedCountC;
extern int DBufUsedCountS;

extern int dbuf_put(DBuf *, const char *, size_t, int);
extern size_t dbuf_get(DBuf *, char *, size_t);
extern char *dbuf_map(DBuf *, size_t *);
extern void dbuf_delete(DBuf *, size_t);
extern int dbuf_getmsg(DBuf *, char *, size_t);
extern void init_dbuf();
extern void count_dbuf_memory(size_t *, size_t *, size_t *, size_t *, int *, int *, int *, int *);
extern int dbuf_startfree_block(int, int, int (*)(int, int));

#endif /* __dbuf_include__ */
