/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_capab.c: CAPAB protocol command
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_capab.c,v 1.26 2002/12/15 21:26:17 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

static struct aCapability {
	char *token;
	long flag;
} Capabilities[] = {
	{ "NOQUIT", CAPAB_NOQUIT },
	{ "BURST", CAPAB_BURST },
	{ "UNCONNECT", CAPAB_UNCONN },
	{ "ZIP", CAPAB_ZIP },
	{ "SSJOIN", CAPAB_SSJOIN },
	{ "SSJv2", CAPAB_SSJv2 },
	{ "DKEY", CAPAB_DKEY },
	{ "NICKIP", CAPAB_NICKIP },
	{ "VHOST", CAPAB_VHOST },
	{ "SNICK", CAPAB_SNICK },
	{ NULL, 0L },
};

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_capab.so",
	"CAPAB protocol",
	4,
	"$Revision: 1.26 $",
	0, NULL, NULL
};
void _module_load() {
	addCommand("CAPAB", m_capab, MAXPARA, MSG_UNREGUSE);
	return;
}
void _module_unload() {
	delCommand("CAPAB", m_capab);
	return;
}
#endif

/*
 * m_capab
 *	parv[0] = sender prefix
 *	parv[n] = capabilities
 */
int m_capab(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	char capab[128];
	struct aCapability *cap;
	int i;

	if (cptr != sptr) {
		return 0;
	}
	for (i = 1; i < parc; i++) {
		strncpy(capab, parv[i], 127);
		capab[127] = '\0';
		for (cap = Capabilities; cap->token != NULL; cap++) {
			if (!irccmp(cap->token, capab)) {
				AddCapab(cptr, cap->flag);
				break;
			}
		}
	}
	return 0;
}
