/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_connect.c: CONNECT command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_connect.c,v 1.18 2002/12/15 21:26:17 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>  
#include <string.h>
#include <fcntl.h>

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_connect.so",
	"/CONNECT command",
	4,
	"$Revision: 1.18 $",
	0, NULL, NULL
};
void _module_load() {
	addCommand("CONNECT", m_connect, MAXPARA, MSG_NONE);
	return;
}
void _module_unload() {
	delCommand("CONNECT", m_connect);
	return;
}
#endif

/*
 * m_connect
 *	parv[0] = sender prefix
 *	parv[1] = servername
 *	parv[2] = port number
 *	parv[3] = remote server
 */
int m_connect(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	int port, tmpport;
	ConfigItem_link *link;
	aClient *acptr;
	char *inpath = NULL;

	if (!IsPrivileged(sptr) || (MyClient(sptr) && ((!OPHasFlag(sptr, OFLAG_LROUTE) && parc <= 3) ||
	  (!OPHasFlag(sptr, OFLAG_GROUTE) && parc > 3)))) {
		sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
		return 0;
	}
	if (parc < 2 || *parv[1] == '\0') {
		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "CONNECT");
		return 0;
	}
	if (hunt_server(cptr, sptr, ":%s CONNECT %s %s :%s", 3, parc, parv) != HUNTED_ISME) {
		return 0;
	}
	if ((acptr = find_server(parv[1], NULL)) != NULL) {
		sendto_one(sptr, ":%s NOTICE %s :*** Connect: server %s already exists from %s.",
			me.name, parv[0], parv[1], acptr->from->name);
		return 0;
	}
	if ((link = find_link(parv[1], "*")) == NULL) {
		sendto_one(sptr, ":%s NOTICE %s :*** Connect: no link configuration found for %s.",
			me.name, parv[0], parv[1]);
		return 0;
	}

	tmpport = port = link->port;
	if (parc > 2 && !BadPtr(parv[2])) {
		if ((port = atoi(parv[2])) <= 0) {
			sendto_one(sptr, ":%s NOTICE %s :*** Connect: illegal port number.",
				me.name, parv[0]);
			return 0;
		}
	}

	sendto_gnotice("%s CONNECT %s %s from %s", MyClient(sptr) ? "Local" : "Remote",
		parv[1], parv[2] && *parv[2] != '\0' ? parv[2] : "", parv[0]);
	sendto_serv_butone(&me, ":%s GNOITCE :%s CONNECT %s %s from %s", me.name,
		MyClient(sptr) ? "Local" : "Remote", parv[0], parv[2] && *parv[2] != '\0' ? parv[2] : "",
		parv[0]);
	ircdlog(LOG_SERVER, "CONNECT from %s: %s %s", parv[0], parv[1], parv[2] && *parv[2] != '\0' ? parv[2] : "*");

	inpath = HasMode(sptr, UMODE_RSTAFF) ? link->fromip : "0.0.0.0";
	link->port = port;
	if (connect_server(link, sptr, NULL)) {
		sendto_one(sptr, ":%s NOTICE %s :*** Connecting to %s[%s].%d.", me.name, parv[0],
			link->servername, inpath, link->port);
	}
	else {
		sendto_one(sptr, ":%s NOTICE %s :*** Couldn't connect to %s[%s].%d.", me.name,
			parv[0], link->servername, inpath, link->port);
	}
	link->port = tmpport;
	return 0;
}
