/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_links.c: LINKS command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_links.c,v 1.16 2002/12/15 21:26:18 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_links.so",
	"/LINKS command",
	4,
	"$Revision: 1.16 $",
	0, NULL, NULL
};
void _module_load() {
	addCommand("LINKS", m_links, MAXPARA, MSG_NONE);
	return;
}
void _module_unload() {
	delCommand("LINKS", m_links);
	return;
}
#endif

/*
 * m_links
 *	parv[0] = sender prefix
 *	parv[1] = servername mask
 * OR
 *	parv[0] = sender prefix
 *	parv[1] = server to query
 *	parv[2] = servername mask
 */
int m_links(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	static time_t last_used = 0L;
	char *mask, *s, *d;
	char clean_mask[(2 * HOSTLEN) + 1];
	aClient *acptr;
	int n;

	if (parc > 2) {
		if (hunt_server(cptr, sptr, ":%s LINKS %s :%s", 1, parc, parv) != HUNTED_ISME) {
			return 0;
		}
		mask = parv[2];
	}
	else {
		mask = parc < 2 ? NULL : parv[1];
	}
	if (!HasMode(sptr, UMODE_OPER)) {
		if (!MyConnect(sptr)) {
			return 0;
		}
		if (last_used + PACETIME_INTENSE > timeofday) {
			sendto_one(sptr, rpl_str(RPL_LOAD2HI), me.name, parv[0]);
			return 0;
		}
		else {
			last_used = timeofday;
		}
		if (SPYNOTICES == 1 && IsPerson(sptr)) {
			sendto_realops_lev(SPY_LEV, "LINKS requested by %s (%s@%s) [%s]", sptr->name,
				sptr->username, MaskedHost(sptr), sptr->user->server);
		}
	}
	if (mask != NULL) {
		s = mask;
		d = clean_mask;
		n = (2 * HOSTLEN) - 2;

		while (*s && n > 0) {
			if ((unsigned char)*s < (unsigned char)' ') {
				*d++ = '^';
				*d++ = (char)((unsigned char)*s + 0x40);
				s++;
				n -= 2;
			}
			else if ((unsigned char)*s > (unsigned char)'~') {
				*d++ = '.';
				s++;
				n--;
			}
			else {
				*d++ = *s++;
				n--;
			}
		}
		*d = '\0';
	}

	collapse(mask);
	for (acptr = client; acptr != NULL; acptr = acptr->next) {
		if (!IsServer(acptr) && !IsMe(acptr)) {
			continue;
		}
		if (!BadPtr(mask) && match(mask, acptr->name)) {
			continue;
		}
		if (HIDEULINES == 1 && !HasMode(sptr, UMODE_OPER) && IsULine(acptr)) {
			continue;
		}
		if (FLATTEN_LINKS == 1 && !HasMode(sptr, UMODE_OPER) && !IsMe(acptr)) {
			sendto_one(sptr, rpl_str(RPL_LINKS), me.name, parv[0], acptr->name,
				me.name, 1, (!BadPtr(acptr->info)) ? acptr->info : "(Unknown Location)");
		}
		else {
			sendto_one(sptr, rpl_str(RPL_LINKS), me.name, parv[0], acptr->name,
				acptr->serv->up, acptr->hopcount,
				(!BadPtr(acptr->info)) ? acptr->info : "(Unknown Location)");
		}
	}
	sendto_one(sptr, rpl_str(RPL_ENDOFLINKS), me.name, parv[0],
		BadPtr(mask) ? "*" : clean_mask);
	return 0;
}
