/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_lusers.c
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_lusers.c,v 1.14 2002/12/15 21:26:18 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_lusers.so",
	"/LUSERS command",
	4,
	"$Revision: 1.14 $",
	0, NULL, NULL
};
void _module_load() {
	addCommand("LUSERS", m_lusers, MAXPARA, MSG_NONE);
	return;
}
void _module_unload() {
	delCommand("LUSERS", m_lusers);
	return;
}
#endif

/*
 * m_lusers
 *	parv[0] = sender prefix
 *	parv[1] = servername
 */
int m_lusers(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	static time_t last_used = 0L;

	if (hunt_server(cptr, sptr, ":%s LUSERS :%s", 1, parc, parv) != HUNTED_ISME) {
		return 0;
	}
	if (!HasMode(sptr, UMODE_OPER)) {
		if (last_used + PACETIME_INTENSE > timeofday) {
			sendto_one(sptr, rpl_str(RPL_LOAD2HI), me.name, parv[0]);
			return 0;
		}
		else {
			last_used = timeofday;
		}
		if (SPYNOTICES == 1 && IsPerson(sptr)) {
			sendto_realops_lev(SPY_LEV, "LUSERS requested by %s (%s@%s) [%s]", sptr->name,
				sptr->username, MaskedHost(sptr), sptr->user->server);
		}
	}
	return send_lusers(cptr, sptr, parc, parv);
}
