/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_sajoin.c: SAJOIN command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_sajoin.c,v 1.9 2002/12/15 21:26:18 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

extern chanMember *add_user_to_channel(aChannel *, aClient *, int);
extern void send_names(aClient *, aChannel *);

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_sajoin.so",
	"/SAJOIN command",
	4,
	"$Revision: 1.9 $",
	0, NULL, NULL
};
void _module_load() {
	addCommand("SAJOIN", m_sajoin, MAXPARA, MSG_NONE);
	return;
}
void _module_unload() {
	delCommand("SAJOIN", m_sajoin);
	return;
}
#endif

/*
 * m_sajoin
 *	parv[0] = sender prefix
 *	parv[1] = channel
 */
int m_sajoin(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aChannel *chptr;
	char *p;

	if (!IsPrivileged(sptr) || !HasMode(sptr, UMODE_SADMIN)) {
		sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);	
		return 0;
	}
	if (parc < 2) {
		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "SAJOIN");
		return 0;
	}
	if ((p = strchr(parv[1], ',')) != NULL) {
		*p = '\0';
	}
	if ((chptr = find_channel(parv[1], NULL)) == NULL) {
		if (!MyClient(sptr)) {
			sendto_realops_lev(DEBUG_LEV, "DESYNCH: client %s!%s@%s has SAJOIN'ed nonexistant channel %s",
				sptr->name, sptr->username, MaskedHost(sptr), parv[1]);
			ircdlog(LOG_ERROR, "DESYNCH: client %s has SAJOIN'ed nonexistant channel %s",
				get_client_name(sptr, TRUE), parv[1]);
		}
		else {
			sendto_one(sptr, err_str(ERR_NOSUCHCHANNEL), me.name, parv[0], parv[1]);
		}
		return 0;
	}
	if (IsMember(sptr, chptr)) {
		return 0;
	}

	sendto_serv_butone(NULL, ":%s GLOBOPS :%s used SAJOIN %s", me.name, sptr->name, chptr->chname);
	sendto_realops("%s used SAJOIN %s", sptr->name, chptr->chname);
	add_user_to_channel(chptr, sptr, 0);
	if (MyClient(sptr)) {
		sendto_ssjoin_servs_butone(0, chptr, cptr, ":%s SJOIN %ld :%s", parv[0], chptr->channelts,
			chptr->chname);
		sendto_ssjoin_servs_butone(1, chptr, cptr, ":%s SJOIN %ld :%s", parv[0], chptr->channelts,
			chptr->chname);
	}
	else {
		sendto_match_servs(chptr, sptr, ":%s SAJOIN :%s", parv[0], chptr->chname);
	}
	sendto_channel_butserv(chptr, sptr, ":%s JOIN :%s", parv[0], chptr->chname);

	if (MyClient(sptr)) {
		del_invite(sptr, chptr);
		if (*chptr->topic != '\0') {
			sendto_one(sptr, rpl_str(RPL_TOPIC), me.name, parv[0], chptr->chname, chptr->topic);
			sendto_one(sptr, rpl_str(RPL_TOPICWHOTIME), me.name, parv[0], chptr->chname,
				chptr->topic_nick, chptr->topic_time);
		}
		send_names(sptr, chptr);
	}
	return 0;
}
