/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_set.c: SET command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_set.c,v 1.14 2002/12/15 21:26:19 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

#ifdef FLUD
extern int flud_num, flud_time, flud_block;
#endif

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_set.so",
	"/SET command",
	4,
	"$Revision: 1.14 $",
	0, NULL, NULL
};
void _module_load() {
	addCommand("SET", m_set, MAXPARA, MSG_NONE);
	return;
}
void _module_unload() {
	delCommand("SET", m_set);
	return;
}
#endif

/*
 * m_set
 *	parv[0] = sender prefix
 *	parv[1] = directive
 *	parv[2] = optional new value
 */
int m_set(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	int val = 0;
	if (!HasMode(sptr, UMODE_ADMIN) || !HasMode(sptr, UMODE_COADMIN)) {
		sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
		return 0;
	}
	if (parc > 2) {
#ifdef FLUD
		if (!mycmp(parv[1], "FLUDNUM")) {
			if ((val = atoi(parv[2])) < 1) {
				sendto_one(sptr, ":%s NOTICE %s :Cannot set FLUDNUM < 1", me.name, parv[0]);
				return 0;
			}
			flud_num = val;
			sendto_realops("%s (%s@%s) has changed FLUDNUM to %d", parv[0], sptr->username,
				MaskedHost(sptr), flud_num);
			return 0;
		}
		if (!mycmp(parv[1], "FLUDTIME")) {
			if ((val = atoi(parv[2])) < 1) {
				sendto_one(sptr, ":%s NOTICE %s :Cannot set FLUDTIME < 1", me.name, parv[0]);
				return 0;
			}
			flud_time = val;
			sendto_realops("%s (%s@%s) has changed FLUDTIME to %d", parv[0], sptr->username,
				MaskedHost(sptr), flud_time);
			return 0;
		}
		if (!mycmp(parv[1], "FLUDBLOCK")) {
			if ((val = atoi(parv[2])) < 0) {
				sendto_one(sptr, ":%s NOTICE %s :Cannot set FLUDBLOCK < 0", me.name, parv[0]);
				return 0;
			}
			if ((flud_block = val) == 0) {
				sendto_realops("%s (%s@%s) has disabled FLUD protection", parv[0], sptr->username,
					MaskedHost(sptr));
			}
			else {
				sendto_realops("%s (%s@%s) has changed FLUDBLOCK to %d", parv[0], sptr->username,
					MaskedHost(sptr));
			}
			return 0;
		}
#endif
		if (ANTI_SPAMBOT_CHECKS == 1) {
			if (!mycmp(parv[1], "SPAMNUM")) {
				if ((val = atoi(parv[2])) < MIN_SPAM_NUM) {
					sendto_one(sptr, ":%s NOTICE %s :Cannot set SPAMNUM < %d", me.name, parv[0],
						MIN_SPAM_NUM);
					return 0;
				}
				spam_num = val;
				sendto_realops("%s (%s@%s) has changed SPAMNUM to %d", parv[0], sptr->username,
					MaskedHost(sptr), spam_num);
				return 0;
			}
			if (!mycmp(parv[1], "SPAMTIME")) {
				if ((val = atoi(parv[2])) < MIN_SPAM_TIME) {
					sendto_one(sptr, ":%s NOTICE %s :Cannot set SPAMTIME < %d", me.name, parv[0],
						MIN_SPAM_NUM);
					return 0;
				}
				spam_time = val;
				sendto_realops("%s (%s@%s) has changed SPAMTIME to %d", parv[0], sptr->username,
					MaskedHost(sptr), spam_time);
				return 0;
			}
		}
	}
	sendto_one(sptr, ":%s NOTICE %s :The following options are available:", me.name, parv[0]);
#ifdef FLUD
	sendto_one(sptr, ":%s NOTICE %s :FLUDNUM(%d) FLUDTIME(%d) FLUDBLOCK(%d)", me.name, parv[0],
		flud_num, flud_time, flud_block);
#endif
	if (ANTI_SPAMBOT_CHECKS == 1) {
		sendto_one(sptr, ":%s NOTICE %s :SPAMNUM(%d) SPAMTIME(%d)", me.name, parv[0], spam_num,
			spam_time);
	}
	return 0;
}
