/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_svsnick.c: SVSNICK command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_svsnick.c,v 1.11 2002/12/15 21:26:20 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_svsnick.so",
	"/SVSNICK command",
	4,
	"$Revision: 1.11 $",
	0, NULL, NULL
};
void _module_load() {
	addCommand("SVSNICK", m_svsnick, MAXPARA, MSG_NONE);
	return;
}
void _module_unload() {
	delCommand("SVSNICK", m_svsnick);
	return;
}
#endif

/*
 * m_svsnick
 *	parv[0] = sender prefix
 *	parv[1] = old nickname
 *	parv[2] = new nickname
 *	parv[3] = TS
 *
 * Implemented from DreamForge by Rastlin.
 */
int m_svsnick(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aClient *acptr, *ocptr;
	char newnick[NICKLEN + 1];

	if (!IsULine(sptr) || parc < 4 || strlen(parv[2]) > NICKLEN) {
		return 0;
	}
	if (hunt_server(cptr, sptr, ":%s SVSNICK %s %s :%s", 1, parc, parv) != HUNTED_ISME) {
		return 0;
	}
	if ((acptr = find_person(parv[1], NULL)) == NULL) {
		return 0;
	}

	strncpyzt(newnick, parv[2], NICKLEN+1);
	if ((acptr = find_client(newnick, NULL)) != NULL) {
		int cnt = 0, rand_suffix = 0;
		char serv_suffix[HOSTLEN + 1], *ptr;

		strncpyzt(serv_suffix, me.name, NICKLEN+1);
		if ((ptr = strchr(serv_suffix, '.')) != NULL) {
			*ptr = '\0';
		}
		do {
			rand_suffix = rand() % 999;
			ircsnprintf(newnick, NICKLEN, "%s-%d[%s]", parv[2], rand_suffix, serv_suffix);
			cnt++;
		} while ((ocptr = find_client(newnick, NULL)) != NULL && cnt < 10);

		if (ocptr != NULL) {
			if (IsUnknown(ocptr)) {
				return exit_client(ocptr, ocptr, &me, "SVSNICK Override");
			}
			else {
				return exit_client(acptr, acptr, &me, "SVSNICK Collide");
			}
		}
	}
	DelMode(acptr, UMODE_REGNICK);
	acptr->tsinfo = atoi(parv[3]);
	acptr->user->last_nick_change = atoi(parv[3]);
	sendto_common_channels(acptr, ":%s NICK :%s", parv[1], newnick);
	add_history(acptr, 1);
	sendto_serv_butone(NULL, ":%s NICK %s :%d", parv[1], newnick, acptr->tsinfo);
	if (*acptr->name != '\0') {
		del_from_client_hash_table(acptr->name, acptr);
	}
	strcpy(acptr->name, newnick);
	add_to_client_hash_table(newnick, acptr);
	return 0;
}
