/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_topic.c: TOPIC command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_topic.c,v 1.10 2002/12/15 21:26:20 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

#ifndef STATIC_MODULES
Module _module_header = {
	"modules/m_topic.so",
	"/TOPIC command",
	4,
	"$Revision: 1.10 $",
	0, NULL, NULL
};
void _module_load() {
	addCommand("TOPIC", m_topic, MAXPARA, MSG_NONE);
	return;
}
void _module_unload() {
	delCommand("TOPIC", m_topic);
	return;
}
#endif

/*
 * m_topic
 *	parv[0] = sender prefix
 *	parv[1] = channel name
 */
int m_topic(aClient *cptr, aClient *sptr, int parc, char *parv[])	
{
	aChannel *chptr = NULL;
	char *topic = NULL, *name, *tnick = sptr->name;
	time_t ts = timeofday;

	if (parc < 2 || *parv[1] == '\0') {
		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "TOPIC");
		return 0;
	}

	name = parv[1];
	if ((chptr = find_channel(name, NULL)) == NULL) {
		sendto_one(sptr, err_str(ERR_NOSUCHCHANNEL), me.name, parv[0], name);
		return 0;
	}
	if (parc == 2) {
		if (!IsMember(sptr, chptr)) {
			sendto_one(sptr, err_str(ERR_NOTONCHANNEL), me.name, parv[0], name);
			return 0;
		}
		else {
			if (*chptr->topic == '\0') {
				sendto_one(sptr, rpl_str(RPL_NOTOPIC), me.name, parv[0], chptr->chname);
			}
			else {
				sendto_one(sptr, rpl_str(RPL_TOPIC), me.name, parv[0], chptr->chname,
					chptr->topic);
				sendto_one(sptr, rpl_str(RPL_TOPICWHOTIME), me.name, parv[0], chptr->chname,
					chptr->topic_nick, chptr->topic_time);
			}
		}
		return 0;
	}
	if (!IsMember(sptr, chptr) && !IsServer(sptr) && !IsULine(sptr)) {
		sendto_one(sptr, err_str(ERR_NOTONCHANNEL), me.name, parv[0], name);
		return 0;
	}
	if (parc > 3 && (!MyConnect(sptr) || IsULine(sptr) || IsServer(sptr))) {
		topic = parc > 4 ? parv[4] : "";
		tnick = parv[2];
		ts = atoi(parv[3]);
	}
	else {
		topic = parv[2];
	}
	if (((!(chptr->mode.mode & MODE_TOPICLIMIT) || is_chan_op(sptr, chptr) || is_half_op(sptr, chptr))
	  || IsULine(sptr) || IsServer(sptr) || HasMode(sptr, UMODE_OPER))) {
		if (IsServer(sptr) && !IsULine(sptr) && chptr->topic_time >= ts && *chptr->topic != '\0') {
			return 0;
		}
		if (HasMode(sptr, UMODE_OPER) && !is_chan_op(sptr, chptr) && !is_half_op(sptr, chptr) &&
		  !IsULine(sptr) && !IsServer(sptr)) {
			sendto_ops("[OperTopic] - [IRCop: %s] - [Channel: %s] - [New Topic: %s]",
				sptr->name, chptr->chname, topic);
		}
		strncpyzt(chptr->topic, topic, TOPICLEN + 1);
		strcpy(chptr->topic_nick, tnick);
		chptr->topic_time = ts;
		sendto_match_servs(chptr, cptr, ":%s TOPIC %s %s %lu :%s", parv[0], chptr->chname,
			chptr->topic_nick, chptr->topic_time, chptr->topic);
		sendto_channel_butserv(chptr, sptr, ":%s TOPIC %s :%s", parv[0], chptr->chname,
			chptr->topic);
	}
	else {
		sendto_one(sptr, err_str(ERR_CHANOPRIVSNEEDED), me.name, parv[0], chptr->chname);
	}
	return 0;
}
