/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * src/support.c: other misc functions
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: support.c,v 1.23 2002/12/16 00:01:49 rageircd Exp $
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "h.h"
#include "memory.h"

extern int errno;

int msg_has_colour(char *message)
{
	char *s;

	if (message != NULL) {
		for (s = message; *s; s++) {
			if (*s == '\003' || *s == '\033') {
				return 1;
			}
		}
	}
	return 0;
}

unsigned char *strip_colour(char *msg)
{
	int nc = 0, col = 0, i = 0, len = 0;
	static unsigned char new_str[4096];

	for (len = strlen(msg); len > 0; msg++, len--) {
		if (col == 1 && ((isdigit(*msg) && nc < 2) || (*msg == ',' && nc < 3))) {
			nc++;
			if (*msg == ',') {
				nc = 0;
			}
		}
		else {
			col = 0;
			if (*msg == '\003' || *msg == '\033') {
				col = 1;
				nc = 0;
			}
			else {
				new_str[i++] = *msg;
			}
		}
	}
	new_str[i] = '\0';
	return new_str;
}

void strip(char *string)
{
	char *s;

	if (string == NULL) {
		return;
	}
	if ((s = strchr(string, '\n')) || (s = strchr(string, '\r'))) {
		*s = '\0';
	}
	return;
}

static time_t atime_exp(char *base, char *ptr)
{
	time_t tmp;
	char *p = ptr, c = *ptr;

	*ptr-- = '\0';
	while (ptr-- > base) {
		if (isalpha(*ptr)) {
			break;
		}
	}
	tmp = atoi(ptr + 1);
	*p = c;
	return tmp;
}

time_t atime(char *xtime)
{
	char *d, *h, *m, *s;
	time_t D, H, M, S;
	int i = 0;

	d = h = m = s = NULL;
	D = H = M = S = 0;

	for (d = xtime; *d; d++) {
		if (isalpha(*d) && i != 1) {
			i = 1;
		}
	}
	if (i == 0) {
		return atol(xtime);
	}

	d = strchr(xtime, 'd');
	h = strchr(xtime, 'h');
	m = strchr(xtime, 'm');
	s = strchr(xtime, 's');

	if (d != NULL) {
		D = atime_exp(xtime, d);
	}
	if (h != NULL) {
		H = atime_exp(xtime, h);
	}
	if (m != NULL) {
		M = atime_exp(xtime, m);
	}
	if (s != NULL) {
		S = atime_exp(xtime, s);
	}
	return ((D * 86400) + (H * 3600) + (M * 60) + S);
}

char *secstodhms(time_t secs)
{
	int dt = 0, ht = 0, mt = 0, st = 0, pos = 0;
	static char dhms[100];
	char *p;

	if (secs > 86400) {
		dt = (secs - (secs % 86400)) / 86400;
		secs -= (dt * 86400);
	}
	if (secs > 3600) {
		ht = (secs - (secs % 3600)) / 3600;
		secs -= (ht * 3600);
	}
	if (secs > 60) {
		mt = (secs - (secs % 60)) / 60;
		secs -= (mt * 60);
	}
	st = (secs >= 0) ? secs : 0;

	*dhms = '\0';
	if (dt != 0) {
		for (p = (char *)my_itoa(dt); *p; p++) {
			dhms[pos++] = *p;
		}
		for (p = (dt == 1) ? " day" : " days"; *p; p++) {
			dhms[pos++] = *p;
		}
	}
	if (ht != 0) {
		if (pos > 0) {
			dhms[pos++] = ',';
			dhms[pos++] = ' ';
		}
		for (p = (char *)my_itoa(ht); *p; p++) {
			dhms[pos++] = *p;
		}
		for (p = (ht == 1) ? " hr" : " hrs"; *p; p++) {
			dhms[pos++] = *p;
		}
	}
	if (mt != 0) {
		if (pos > 0) {
			dhms[pos++] = ',';
			dhms[pos++] = ' ';
		}
		for (p = (char *)my_itoa(mt); *p; p++) {
			dhms[pos++] = *p;
		}
		for (p = (mt == 1) ? " min" : " mins"; *p; p++) {
			dhms[pos++] = *p;
		}
	}
	if (st != 0) {
		if (pos > 0) {
			dhms[pos++] = ',';
			dhms[pos++] = ' ';
		}
		for (p = (char *)my_itoa(st); *p; p++) {
			dhms[pos++] = *p;
		}
		for (p = (st == 1) ? " sec" : " secs"; *p; p++) {
			dhms[pos++] = *p;
		}
	}
	if (pos == 0) {
		for (p = "0 secs"; *p; p++) {
			dhms[pos++] = *p;
		}
	}
	dhms[pos] = '\0';
	return dhms;
}

#ifndef HAVE_STRTOKEN
char *strtoken(char **save, char *str, char *fs)
{
	char *pos = *save;
	char *tmp;

	if (str) {
		pos = str;
	}
	while (pos && *pos && strchr(fs, *pos)) {
		pos++;
	}
	if (!pos || !*pos) {
		return (pos = *save = NULL);
	}

	tmp = pos;
	while (*pos && !strchr(fs, *pos)) {
		pos++;
	}
	if (*pos) {
		*pos++ = '\0';
	}
	else {
		pos = NULL;
	}
	*save = pos;
	return (tmp);
}
#endif

#ifndef HAVE_STRERROR
char *strerror(int err_no)
{
#if !defined(__FreeBSD__) && !defined(__NetBSD__)
	extern char *str_errlist[];
	extern int sys_nerr;
#endif
	static char buff[40];
	char *errp = (err_no > sys_nerr) ? NULL : sys_errorlist[err_no]);
	if (!errp) {
		ircsprintf(buff, "Unknown error %d", err_no);
		errp = buff;
	}
	return errp;
}
#endif

char *inetntoa(char *in)
{
	static char buf[16];
	u_char *s = (u_char *)in;
	int a, b, c, d;

	a = (int)*s++;
	b = (int)*s++;
	c = (int)*s++;
	d = (int)*s++;
	ircsprintf(buf, "%d.%d.%d.%d", a, b, c, d);
	return buf;
}

#ifndef HAVE_INET_NETOF
int inet_netof(struct in_addr in)
{
	int addr = in.s_net;

	if (!(addr & 0x80)) {
		return (int)in.s_net;
	}
	if (!(addr & 0x40)) {
		return ((int)in.s_net * 256 + in.s_host);
	}
	return ((int)in.s_net * 256 + in.s_host * 256 + in.s_lh);
}
#endif

char *my_itoa(int d)
{
	static char buf[128];
	ircsprintf(buf, "%d", d);
	return buf;
}

int dgets(int fd, char *buf, int num)
{
	static char dgbuf[8192], *head = dgbuf, *tail = dgbuf;
	char *s, *t;
	int n, nr;

	if (head == tail) {
		*head = '\0';
	}
	if (num == 0) {
		head = tail = dgbuf;
		*head = '\0';
		return 0;
	}
	if (num > sizeof(dgbuf) - 1) {
		num = sizeof(dgbuf) - 1;
	}
	for (;;)
	{
		if (head > dgbuf) {
			for (nr = tail - head, s = head, t = dgbuf; nr > 0; nr--) {
				*t++ = *s++;
			}
			tail = t;
			head = dgbuf;
		}
		if (head < tail && ((s = strchr(head, '\n')) || (s = strchr(head, '\r'))) && s < tail) {
			n = MIN(s - head + 1, num);
			memcpy(buf, head, n);
			head += n;
			if (head == tail) {
				head = tail = dgbuf;
			}
			return n;
		}
		if (tail - head >= num) {
			n = num;
			memcpy(buf, head, n);
			if ((head += n) == tail) {
				head = tail = dgbuf;
			}
			return n;
		}

		n = sizeof(dgbuf) - (tail - dgbuf) - 1;
		if ((nr = read(fd, tail, n)) == -1) {
			head = tail = dgbuf;
			return -1;
		}
		if (nr == 0) {
			if (tail > head) {
				n = MIN(tail - head, num);
				memcpy(buf, head, n);
				if ((head += n) == tail) {
					head = tail = dgbuf;
				}
				return n;
			}
			head = tail = dgbuf;
			return 0;
		}

		tail += nr;
		*tail = '\0';

		for (t = head; (s = strchr(t, '\n'));) {
			if ((s > head) && (s > dgbuf)) {
				t = s - 1;
				for (nr = 0; *t == '\\'; nr++) {
					t--;
				}
				if (nr & 1) {
					t = s + 1;
					s--;
					nr = tail - t;
					while (nr--) {
						*s++ = *t++;
					}
					tail -= 2;
					*tail = '\0';
				}
				else {
					s++;
				}
			}
			else {
				s++;
			}
			t = s;
		}
		*tail = '\0';
	}
}
